@@ -896,7 +896,7 @@ class SaleLine(ModelSQL, ModelView):
         states={
             'invisible': Not(In(Eval('type'), ['line', 'subtotal'])),
             'readonly': Not(Bool(Eval('_parent_sale'))),
-        }, on_change_with=['type', 'quantity', 'unit_price',
+        }, on_change_with=['type', 'quantity', 'unit_price', 'unit',
             '_parent_sale.currency']), 'get_amount')
     description = fields.Text('Description', size=None, required=True)
     note = fields.Text('Note')
@@ -1169,6 +1169,8 @@ class SaleLine(ModelSQL, ModelView):
         ignored_ids = set(
             l.id for i in line.sale.invoices_ignored for l in i.lines)
         for invoice_line in line.invoice_lines:
+            if invoice_line.type != 'line':
+                continue
             if ((invoice_line.invoice and
                     invoice_line.invoice.state != 'cancel') or
                 invoice_line.id in ignored_ids):
